#include "tableprint.hpp"
#include <iomanip>
#include <algorithm>
#include <locale>
#include <cwchar>
#include <string>
#include <sstream>
#include <codecvt>
#include <iostream>

// Helper function to replace all occurrences of a substring in a regular string
std::wstring replace_all(const std::string& str, const std::string& from, const std::wstring& to) {
    std::wstring result;
    size_t start_pos = 0;
    size_t find_pos;
    
    while ((find_pos = str.find(from, start_pos)) != std::string::npos) {
        result += std::wstring(str.begin() + start_pos, str.begin() + find_pos);
        result += to;
        start_pos = find_pos + from.length();
    }
    result += std::wstring(str.begin() + start_pos, str.end());
    return result;
}

// Helper function to replace all occurrences of a substring in a wide string
std::wstring replace_all(const std::wstring& str, const std::string& from, const std::wstring& to) {
    std::wstring result;
    size_t start_pos = 0;
    size_t find_pos;
    std::wstring from_w(from.begin(), from.end());
    
    while ((find_pos = str.find(from_w, start_pos)) != std::wstring::npos) {
        result += str.substr(start_pos, find_pos - start_pos);
        result += to;
        start_pos = find_pos + from_w.length();
    }
    result += str.substr(start_pos);
    return result;
}

tableprint::tableprint(const std::string title) : title(title) {
    // Set locale for wide character support
    std::setlocale(LC_ALL, "");
}

tableprint::~tableprint() {}

void tableprint::set_title(const std::string title) {
    this->title = title;
}

void tableprint::add_row(const std::vector<std::string>& row) {
    rows.push_back(row);
}

void tableprint::print() {
    if (rows.empty()) return;

    // Calculate column widths
    std::vector<size_t> col_widths(rows[0].size(), 0);
    for (const auto& row : rows) {
        for (size_t i = 0; i < row.size(); ++i) {
            std::string cell = row[i];
            // Count occurrences of :tick: and :cross:
            size_t tick_count = std::count(cell.begin(), cell.end(), ':') / 2;
            // Adjust width: original length minus 5 for each :tick: or :cross: (since they become 1 char)
            col_widths[i] = std::max(col_widths[i], cell.length() - (tick_count * 5));
        }
    }

    // Calculate total table width
    size_t total_width = 0;
    for (size_t width : col_widths) {
        total_width += width + 2; // +2 for padding
    }
    total_width += col_widths.size() - 1; // Add space for vertical borders

    // Print title if it exists
    if (!title.empty()) {
        std::cout << "\033[90m"; // Dark grey color for borders
        std::cout << "┌";
        for (size_t i = 0; i < total_width; ++i) {
            std::cout << "-";
        }
        std::cout << "┐" << std::endl;
        
        std::cout << "│" << "\033[1;37m"; // White color for title
        size_t padding = (total_width - title.length()) / 2;
        for (size_t i = 0; i < padding; ++i) {
            std::cout << " ";
        }
        std::cout << title;
        for (size_t i = 0; i < total_width - title.length() - padding; ++i) {
            std::cout << " ";
        }
        std::cout << "\033[90m│" << std::endl;
        
        // Use └─┴─┘ for bottom of title box to connect with table
        std::cout << "├";
        for (size_t i = 0; i < rows[0].size(); ++i) {
            for (size_t j = 0; j < col_widths[i] + 2; ++j) {
                std::cout << "-";
            }
            if (i < rows[0].size() - 1) std::cout << "┬";
        }
        std::cout << "┤" << std::endl;
    } else {
        // Print top border if no title
        std::cout << "\033[90m"; // Dark grey color for borders
        std::cout << "┌";
        for (size_t i = 0; i < rows[0].size(); ++i) {
            for (size_t j = 0; j < col_widths[i] + 2; ++j) {
                std::cout << "-";
            }
            if (i < rows[0].size() - 1) std::cout << "┬";
        }
        std::cout << "┐" << std::endl;
    }

    // Print header
    std::cout << "│";
    std::cout << "\033[1;36m"; // Cyan color for header
    for (size_t i = 0; i < rows[0].size(); ++i) {
        std::cout << " " << std::setw(col_widths[i]) << std::left << rows[0][i] << " ";
        if (i < rows[0].size() - 1) {
            std::cout << "\033[90m│\033[1;36m"; // Border color then back to cyan
        } else {
            std::cout << "\033[90m│"; // Just border color for last column
        }
    }
    std::cout << std::endl;

    // Print header separator
    std::cout << "├";
    for (size_t i = 0; i < rows[0].size(); ++i) {
        for (size_t j = 0; j < col_widths[i] + 2; ++j) {
            std::cout << "-";
        }
        if (i < rows[0].size() - 1) std::cout << "┼";
    }
    std::cout << "┤" << std::endl;

    // Print rows
    for (size_t row_idx = 1; row_idx < rows.size(); ++row_idx) {
        const auto& row = rows[row_idx];
        std::cout << "│";
        for (size_t i = 0; i < row.size(); ++i) {
            std::string cell = row[i];
            // Replace :tick: and :cross: with colored symbols
            std::string processed_cell;
            size_t pos = 0;
            while (pos < cell.length()) {
                if (cell.compare(pos, 6, ":tick:") == 0) {
                    processed_cell += "\033[1;32m✓\033[0m";
                    pos += 6;
                } else if (cell.compare(pos, 7, ":cross:") == 0) {
                    processed_cell += "\033[1;31m✗\033[0m";
                    pos += 7;
                } else {
                    processed_cell += cell[pos];
                    pos++;
                }
            }
            
            // Set the appropriate color for the row
            if (row_idx % 2 == 1) {
                std::cout << " " << "\033[38;5;142m";
                if (cell.find(":tick:") != std::string::npos || cell.find(":cross:") != std::string::npos) {
                    std::cout << "  " << processed_cell << "  ";
                } else {
                    std::cout << std::setw(col_widths[i]) << std::left << processed_cell;
                }
                std::cout << "\033[90m" << " │";
            } else {
                std::cout << " " << "\033[38;5;250m";
                if (cell.find(":tick:") != std::string::npos || cell.find(":cross:") != std::string::npos) {
                    std::cout << "  " << processed_cell << "  ";
                } else {
                    std::cout << std::setw(col_widths[i]) << std::left << processed_cell;
                }
                std::cout << "\033[90m" << " │";
            }
        }
        std::cout << std::endl;

        // Print row separator if not the last row
        if (row_idx < rows.size() - 1) {
            std::cout << "├";
            for (size_t i = 0; i < rows[0].size(); ++i) {
                for (size_t j = 0; j < col_widths[i] + 2; ++j) {
                    std::cout << "-";
                }
                if (i < rows[0].size() - 1) std::cout << "┼";
            }
            std::cout << "┤" << std::endl;
        }
    }

    // Print bottom border
    std::cout << "└";
    for (size_t i = 0; i < rows[0].size(); ++i) {
        for (size_t j = 0; j < col_widths[i] + 2; ++j) {
            std::cout << "-";
        }
        if (i < rows[0].size() - 1) std::cout << "┴";
    }
    std::cout << "┘" << "\033[0m" << std::endl; // Reset color
} 